<div class="tablature-table">

  <div class="config">
    <span class="ajax-url">{{ ajax_url }}</span>
    <div class="messages">
      <span class="results">{{ _('results') }}</span>
      <span class="sort-by">{{ _('Sort by this column') }}</span>
      <span class="filter-by">{{ _('Filter by this column') }}</span>
      <span class="clear-filter">{{ _('Clear selection') }}</span>
    </div>
  </div>

  <header class="row">
    <div class="col-md-5">
      <div class="row">
        <form class="col-md-7 search-form">
          <div class="input-group">
            <input type="text" placeholder="{{ _('Search…') }}" class="form-control" />
            <span class="input-group-btn">
              <button type="submit" class="btn btn-default">
                <i class="fa fa-search"></i>
              </button>
            </span>
          </div>
        </form>
        <div class="col-md-5">
          <span class="count"></span>
          <span class="spinner pull-right">
            <i class="fa fa-spinner fa-spin"></i>
          </span>
        </div>
      </div>
    </div>

    <nav class="col-md-7 text-right">
      <ul class="pagination pagination-sm"></ul>
    </nav>
  </header>

  <div class="table-responsive">
    <table class="table table-condensed table-striped table-bordered">
      <thead></thead>
      <tbody></tbody>
    </table>
  </div>

</div>
