{% extends 'base.html' %}


{% block title %}
  {{ verbose_name_plural|capitalize }}
{% endblock %}


{% block css %}
  {{ super() }}
  <link rel="stylesheet" type="text/less" href="{{ static('less/tablature.less') }}" />
{% endblock %}


{% block content %}
  {% include 'tablature/includes/table.html' %}
{% endblock %}


{% block js %}
  {{ super() }}
  <script src="{{ static('js/tablature.js') }}"></script>
{% endblock %}
